import click
from data.main import connect
import pandas as pd
from lxml import etree
from pathlib import Path
import os
import csv

def label_to_int(rating:str) -> int:

    mapping = {
        'left' : 0,
        'left-center' : 1,
        'center' : 2,
        'right-center' : 3,
        'right' : 4,
        'allsides' : -1,
    }

    return mapping[rating]

def int_to_label(class_id: int) -> str:
    mapping = {
            0 : 'left',
            1 : 'left-center',
            2 : 'center',
            3 : 'right-center',
            4 : 'right',
            -1 : 'allsides',
    }
    return mapping[class_id]


@click.command(name="bias:normalize")
def normalize() -> None:
    DB = connect()

    DB.sql("""
        CREATE OR REPLACE TABLE publisher_bias AS
        WITH cte AS (
            SELECT
                p.id as publisher_id
                ,b.id as bias_id
                ,b.bias as label
                ,JARO_WINKLER_SIMILARITY(LOWER(p.name), LOWER(b.publisher)) as similarity
            FROM bias_ratings b
            JOIN top.publishers p
            ON JARO_WINKLER_SIMILARITY(LOWER(p.name), LOWER(b.publisher)) > 0.95
        ),ranked AS (
            SELECT
                publisher_id
                ,bias_id
                ,label
                ,similarity
                ,ROW_NUMBER() OVER(PARTITION BY publisher_id ORDER BY similarity DESC) AS rn
            FROM cte
        )
        SELECT
            publisher_id
            ,label
            ,bias_id
        FROM ranked
        WHERE ranked.rn = 1
    """)

    mapping = [
            {'label' :'left' , 'ordinal': -2},
            {'label' :'left-center' , 'ordinal': -1},
            {'label' :'center' , 'ordinal': 0},
            {'label' :'right-center' , 'ordinal': 1},
            {'label' :'right' , 'ordinal': 2},
    ]
    mapping = pd.DataFrame(mapping)

    DB.query("alter table bias_ratings add column ordinal int")

    DB.query("""
        update bias_ratings b
        set ordinal = o.ordinal
        FROM mapping o
        WHERE o.label = b.bias
    """)


@click.command(name='bias:parse')
def parse() -> None:
    """parse the save html page of allslides.com bias ratings into a normalized csv file"""
    DB = connect()
    DATA_DIR = Path(os.environ['DATA_MINING_DATA_DIR'])
    bias_html = DATA_DIR / 'allsides.html'

    parser = etree.HTMLParser()
    tree = etree.parse(str(bias_html), parser)
    root = tree.getroot()
    rows = root.xpath('//table[contains(@class,"views-table")]/tbody/tr')

    ratings = []
    for row in rows:
        rating = dict()
        publisher = row.xpath('./td[contains(@class, "source-title")]/a')[0].text
        rating['publisher'] = publisher

        bias = row.xpath('./td[contains(@class, "views-field-field-bias-image")]/a')[0].get('href')
        bias = bias.split('/')[-1]
        rating['bias'] = bias

        agree = row.xpath('.//span[contains(@class, "agree")]')[0].text
        disagree = row.xpath('.//span[contains(@class, "disagree")]')[0].text

        rating['agree'] = int(agree)
        rating['disagree'] = int(disagree)
        ratings.append(rating)
    df = pd.DataFrame(ratings)
    df.to_csv(DATA_DIR / 'bias_ratings.csv', sep="|", index=False, quoting=csv.QUOTE_NONNUMERIC)

@click.command(name="bias:load")
def load() -> None:
    DB = connect()
    DATA_DIR = Path(os.environ['DATA_MINING_DATA_DIR'])
    f = str(DATA_DIR / "bias_ratings.csv")

    DB.sql(f"""
        CREATE TABLE bias_ratings as 
        select 
            row_number() over(order by b.publisher) as id
            ,b.*
        from read_csv_auto('{f}') b
    """)

@click.command('bias:export')
def export():
    data_path = Path(os.environ['DATA_MINING_DATA_DIR'])

    DB = connect()
    all_bias = DB.query("""
        SELECT
            id as bias_id
            ,publisher as name
            ,bias as label
        FROM bias_ratings
        ORDER by agree desc
    """)
    all_bias.df().to_csv(data_path / 'TMP_publisher_bias.csv', sep="|", index=False)
    mapped_bias = DB.query("""
        SELECT
            p.id as publisher_id
            ,p.name as name
            ,p.tld as tld
            ,b.label as bias
            ,b.bias_id as bias_id
        FROM top.publishers p
        LEFT JOIN publisher_bias b
        ON b.publisher_id = p.id
    """)
    mapped_bias.df().to_csv(data_path / 'TMP_publisher_bias_to_load.csv', sep="|", index=False)
    DB.close()

@click.command('bias:import-mapped')
def import_mapped():
    data_path = Path(os.environ['DATA_MINING_DATA_DIR'])
    table_name = "top.publisher_bias"

    DB = connect()
    df = pd.read_csv(data_path / 'TMP_publisher_bias_to_load.csv', sep="|")

    DB.query(f"""
        CREATE OR REPLACE TABLE {table_name} AS
        SELECT
            publisher_id AS publisher_id
            ,cast(bias_id AS int) as bias_id
        FROM df
        WHERE bias_id IS NOT NULL
    """)
    print(f"created table: {table_name}")

