import click
from data.main import connect
import pandas as pd
import numpy as np
import seaborn as sns
import matplotlib.pyplot as plt


@click.command('links:create-table')
def create_table():

    table_name = "top.link_edges"
    DB = connect()
    DB.query(f"""
        CREATE OR REPLACE TABLE {table_name} AS
        with cte as(
            SELECT 
                s.publisher_id as parent_id
                ,r.publisher_id as child_id
                ,count(1) as links
            FROM top.stories s
            JOIN top.related_stories r
            ON s.id = r.parent_id
            group by 
                s.publisher_id
                ,r.publisher_id
        )
        SELECT
            cte.parent_id
            ,cte.child_id
            ,cte.links as links
            ,cast(cte.links as float) / sum(cte.links) over(partition by cte.parent_id) as normalized
            ,case when cte.links > 0 then 1 else 0 end as onehot
        FROM cte
        WHERE cte.child_id in (
            SELECT
                distinct parent_id
            FROM cte
        )
        AND cte.parent_id in (
            SELECT
                distinct child_id
            FROM cte
        )
    """)
    DB.close()

    DB = connect()
    DB.query("""
        SELECT
        *
        ,-log10(links)
        --distinct parent_id
        FROM top.link_edges e
        WHERE e.parent_id = 238
    """)
    DB.close()
    print(f"created {table_name}")

@click.command('links:create-pca')
@click.option('--source', type=click.Choice(['links', 'normalized', 'onehot']), default='links')
def create_pca(source):
    """create 2D pca labels"""

    from sklearn.decomposition import PCA

    table_name = f"top.publisher_pca_{source}"
    DB = connect()
    pub = DB.query("""
        SELECT
            *
        FROM top.publishers
    """).df()
    df = DB.query(f"""
        SELECT
            parent_id
            ,child_id
            ,{source} as links
        FROM top.link_edges
    """).df()
    DB.close()
    pivot = df.pivot(index='parent_id', columns='child_id', values='links').fillna(0)

    svd = PCA(n_components=2)
    svd_out = svd.fit_transform(pivot)

    out = pivot.reset_index()[['parent_id']]
    out['first'] = svd_out[:, 0]
    out['second'] = svd_out[:, 1]
    out = pd.merge(out, pub, left_on='parent_id', right_on='id')

    DB = connect()
    DB.query(f"""
        CREATE OR REPLACE TABLE {table_name} AS
        SELECT
            out.id as publisher_id
            ,out.first as first
            ,out.second as second
        FROM out
    """)
    DB.close()
    print(f"created {table_name}")


@click.command('links:create-clusters')
@click.option('--source', type=click.Choice(['links', 'normalized', 'onehot']), default='links')
def create_clusters(source):
    from sklearn.cluster import KMeans

    table_name = f"top.publisher_clusters_{source}"
    DB = connect()
    df = DB.query(f"""
        SELECT
            parent_id
            ,child_id
            ,{source} as links
        FROM top.link_edges
    """).df()
    pub = DB.query("""
        SELECT
            *
        FROM top.publishers
    """).df()
    DB.close()
    pivot = df.pivot(index='parent_id', columns='child_id', values='links').fillna(0)


    k = 8
    kmeans = KMeans(n_clusters=k, n_init="auto")
    pred = kmeans.fit_predict(pivot)
    out = pivot.reset_index()[['parent_id']]
    out['label'] = pred
    out = pd.merge(out, pub, left_on='parent_id', right_on='id')
    new_table = out[['id', 'label']]

    DB = connect()
    DB.query(f"""
        CREATE OR REPLACE TABLE {table_name} AS
        SELECT
            n.id as publisher_id
            ,n.label as label
        FROM new_table n
    """)
    DB.close()
    print(f"created {table_name}")

def to_matrix():
    """returns an adjacency matrix of publishers to publisher link frequency"""

    DB = connect()

    bias_map = pd.DataFrame([
        {'label' :'left', 'value' : 0},
        {'label' :'left-center', 'value' : 1},
        {'label' :'center', 'value' : 2},
        {'label' :'right-center', 'value' : 3},
        {'label' :'right', 'value' : 4},
        {'label' :'allsides', 'value' : -1},
    ])

    bias = DB.sql("""
        SELECT
            b.id
            ,b.label
            ,m.value
        FROM publisher_bias b
        JOIN bias_map m
        ON b.label = m.label
        WHERE value != -1
    """).df()

    pub = DB.sql("""
            select 
                p.id
                ,p.name
                ,p.url
            from publishers p
    """).df()

    edges = DB.sql("""
        WITH total as (
            SELECT
                s.publisher_id as id
                ,COUNT(1) as stories
            FROM stories s
            GROUP BY 
                s.publisher_id
        ), p as (
            SELECT
                p.id
                ,stories
            FROM publishers p
            LEFT JOIN total t
            ON t.id = p.id
            WHERE t.stories >= 20
        ), cte as (
            SELECT 
                r.publisher_id as child_id
                ,s.publisher_id as parent_id
                ,count(1) as links
            FROM related_stories r
            JOIN stories s
            ON s.id = r.parent_id
            group by 
                s.publisher_id
                ,r.publisher_id
        )
        SELECT
            p.id as parent_id
            ,cte.child_id
            ,links
        FROM p
        left JOIN cte
        ON p.id = cte.parent_id
    """).df()

    # only keep values that have more than 1 link
    test = edges[edges['links'] > 2].pivot(index='parent_id', columns='child_id', values='links').fillna(0).reset_index()
    edges.dropna().pivot(index='parent_id', columns='child_id', values='links').fillna(0)
    pd.merge(adj, pub, how='left', left_on='parent_id', right_on='id')
    adj = edges.pivot(index='parent_id', columns='child_id', values='links').fillna(0)
    adj.values.shape

    
    out = pd.DataFrame(adj.index.values, columns=['id'])
    out = pd.merge(out, pub, how='left', on='id')
    return out

@click.command('links:analysis')
def analysis():
    from sklearn.decomposition import PCA, TruncatedSVD
    from sklearn.cluster import MiniBatchKMeans
    adj = to_matrix()
    pca = PCA(n_components=4)
    pca_out = pca.fit_transform(adj)

    svd = TruncatedSVD(n_components=2, n_iter=7, random_state=42)
    svd_out = svd.fit_transform(adj)

    x = svd_out[:, 0]
    y = svd_out[:, 1]

    x = pca_out[:, 0]
    y = pca_out[:, 1]
    sns.scatterplot(x=x, y=y)
    plt.show()

    kmeans = MiniBatchKMeans(n_clusters=5, random_state=0, batch_size=6, n_init="auto")
    pred = kmeans.fit_predict(pca_out)

    sns.scatterplot(x=x, y=y, hue=pred)
    plt.show()

    sns.scatterplot(x=pub['first'], y=pub['second'], hue=pub['bias'])
    plt.show()
