import click
from data.main import connect
import os
import seaborn as sns
import matplotlib.pyplot as plt
from pathlib import Path
import numpy as np

out_dir = Path(os.getenv('DATA_MINING_DOC_DIR')) / 'figures'

@click.command('plot:articles-per-year')
def articles_per_year():
    filename = 'articles_per_year.png'

    DB = connect()
    data = DB.query("""
        select
            year(published_at) as year
            ,count(1) as stories
        from stories
        group by 
            year(published_at)
    """).df()
    DB.close()

    ax = sns.barplot(x=data.year, y=data.stories, color='tab:blue')
    ax.tick_params(axis='x', rotation=90)
    ax.set(title="count of articles per year", ylabel="count of stories (#)")
    plt.tight_layout()
    plt.savefig(out_dir / filename)

@click.command('plot:distinct-publishers')
def distinct_publishers():
    filename = 'distinct_publishers.png'

    DB = connect()
    data = DB.query("""
        select
            year(published_at) as year
            ,count(distinct publisher_id) as publishers
        from stories
        group by 
            year(published_at)
    """).df()
    DB.close()

    ax = sns.barplot(x=data.year, y=data.publishers, color='tab:blue')
    ax.tick_params(axis='x', rotation=90)
    ax.set(title="count of publishers per year", ylabel="count of publishers (#)")
    plt.tight_layout()
    plt.savefig(out_dir / filename)
    plt.close()

@click.command('plot:stories-per-publisher')
def stories_per_publisher():
    filename = 'stories_per_publisher.png'

    DB = connect()
    data = DB.query("""
        with cte as (
        select
            publisher_id
            ,year(published_at) as year
            ,count(1) as stories
        from stories
        group by 
            publisher_id
            ,year(published_at)
        ) , agg as (
            select
                publisher_id
                ,avg(stories) as stories_per_year
                ,case 
                    when avg(stories) < 2 then 2
                    when avg(stories) < 4 then 4
                    when avg(stories) < 8 then 8
                    when avg(stories) < 16 then 16
                    when avg(stories) < 32 then 32
                    when avg(stories) < 64 then 64
                    when avg(stories) < 128 then 128
                    else 129
                end as max_avg
            from cte
            group by 
                publisher_id
        )
        select
            max_avg
            ,count(1) as publishers
        from agg
        group by
            max_avg
    """).df()
    DB.close()

    ax = sns.barplot(x=data.max_avg, y=data.publishers, color='tab:blue')
    ax.set(title="histogram of publisher stories per year", ylabel="count of publishers (#)", xlabel="max average stories / year")
    plt.tight_layout()
    plt.savefig(out_dir / filename)
    plt.close()


@click.command('plot:top-publishers')
def top_publishers():
    """plot top publishers over time"""

    filename = 'top_publishers.png'

    DB = connect()
    data = DB.query("""
        select
            p.tld
            ,year(published_at) as year
            ,count(1) as stories
        from (
            select
                p.tld
                ,p.id
            from top.publishers p
            join top.stories s
            on s.publisher_id = p.id
            group by
                p.tld
                ,p.id
            order by count(1) desc
            limit 20
        ) p
        join top.stories s
        on s.publisher_id = p.id
        group by 
            p.tld
            ,year(published_at)
        order by count(distinct s.id) desc
    """).df()
    DB.close()
    
    pivot = data.pivot(columns='year', index='tld', values='stories')
    ax = sns.heatmap(pivot, cmap="crest")
    ax.set(title="top 20 publishers (by tld)", ylabel="tld", xlabel="stories / year (#)")
    plt.tight_layout()
    plt.savefig(out_dir / filename)
    plt.close()


@click.command('plot:common_tld')
def common_tld():
    import dataframe_image as dfi
    filename = 'common_tld.png'

    DB = connect()
    data = DB.query("""
        select
            split_part(url, '.', -1) as tld
            ,count(1) as publishers
            ,case when count(1) < 20
                then string_agg(distinct url, '\t')
                else NULL
                end as urls
        from publishers
        group by
            split_part(url, '.', -1)
        order by 
            count(1) desc
    """).df()
    DB.close()
    data[:15][['tld', 'publishers']].style.hide(axis="index").export_png(out_dir / filename, table_conversion='matplotlib')

def stats():

    # raw
    DB.query("""
        SELECT
            'total stories' as key
            ,COUNT(1) as value
        FROM stories
        UNION
        SELECT
            'total related' as key
            ,COUNT(1) as value
        FROM related_stories
        UNION
        SELECT
            'top level domains' as key
            ,COUNT(distinct tld) as value
        FROM stories
        UNION
        SELECT
            'publishers' as key
            ,COUNT(1) as value
        FROM publishers
        UNION
        SELECT
            'authors' as key
            ,COUNT(distinct author) as value
        FROM stories
        UNION
        SELECT
            'min year' as key
            ,min(year(published_at)) as value
        FROM stories
        UNION
        SELECT
            'max year' as key
            ,max(year(published_at)) as value
        FROM stories
    """).df().to_markdown(index=False)

    # selected
    DB.query("""
        SELECT
            'total stories' as key
            ,COUNT(1) as value
        FROM top.stories
        UNION
        SELECT
            'total related' as key
            ,COUNT(1) as value
        FROM top.related_stories
        UNION
        SELECT
            'top level domains' as key
            ,COUNT(distinct tld) as value
        FROM top.stories
        UNION
        SELECT
            'publishers' as key
            ,COUNT(1) as value
        FROM top.publishers
        UNION
        SELECT
            'authors' as key
            ,COUNT(distinct author) as value
        FROM top.stories
        UNION
        SELECT
            'min year' as key
            ,min(year(published_at)) as value
        FROM top.stories
        UNION
        SELECT
            'max year' as key
            ,max(year(published_at)) as value
        FROM top.stories
    """).df().to_markdown(index=False)

@click.command('plot:bias-stats')
def bias_stats():
    import dataframe_image as dfi
    filename = 'bias_stats.png'

    DB = connect()

    df = DB.query("""
        SELECT
            string_agg(distinct bias)
        FROM bias_ratings
    """).df()

    df.keys()

    df = DB.query("""
        SELECT
            'publishers' as metric
            ,count(1) as value
        FROM bias_ratings
        UNION
        SELECT
            'labels' as metric
            ,count(distinct bias) as value
        FROM bias_ratings
        UNION
        SELECT
            'right' as metric
            ,count(1) as value
        FROM bias_ratings
        WHERE bias in ('right', 'right-center')
        UNION
        SELECT
            'left' as metric
            ,count(1) as value
        FROM bias_ratings
        WHERE bias in ('left', 'left-center')
        UNION
        SELECT
            'center' as metric
            ,count(1) as value
        FROM bias_ratings
        WHERE bias in ('center')
        UNION
        SELECT
            'agree_range' as metric
            ,'[' 
                || min(cast(agree as float) / (agree + disagree)) 
                || '-' 
                || max(cast(agree as float) / (agree + disagree)) 
                || ']'
                as value
        FROM bias_ratings
        WHERE bias in ('center')
    """).df()
    DB.close()
    print(df.to_markdown(index=False))
