import click
from data.main import connect
import os
from pathlib import Path
import seaborn as sns
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd

out_path = Path(os.getenv('DATA_MINING_DOC_DIR')) / 'figures'

@click.command('plot:emotion-over-time')
def emotion_over_time():
    filename = "emotion_over_time.png"
    DB = connect()

    emotions = DB.sql("""
        SELECT
            date_trunc('year', s.published_at) AS year
            ,e.label AS emotion
            ,count(1) AS stories
        FROM top.stories s
        JOIN story_emotions se
        ON s.id = se.story_id
        JOIN emotions e
        ON e.id = se.emotion_id
        GROUP by
            date_trunc('year', s.published_at)
            ,e.label
    """).df()
    DB.close()

    ax = sns.scatterplot(x=emotions['year'], y=emotions['stories'], hue=emotions['emotion'])
    ax.set(title="title emotions over years", xlabel="year", ylabel="stories (#)")
    plt.savefig(out_path / filename)
    print(f"saved: {filename}")

@click.command('plot:emotion-regression')
def emotion_regression():
    from sklearn import linear_model
    from sklearn.model_selection import train_test_split
    from sklearn.metrics import ConfusionMatrixDisplay

    filename = "emotion_regression.png"


    DB = connect()
    emotions = DB.query("""
        SELECT
        label
        FROM emotions e
    """).df()['label'].to_list()
    DB.close()

    DB = connect()
    df = DB.sql(f"""
        SELECT
            epoch(date_trunc('yearweek', s.published_at)) AS date
            ,e.id AS emotion_id
            ,p.id as publisher_id
            ,count(1) AS stories
        FROM top.stories s
        JOIN top.publishers p
        ON p.id = s.publisher_id
        JOIN story_emotions se
        ON s.id = se.story_id
        JOIN emotions e
        ON e.id = se.emotion_id
        GROUP by
            epoch(date_trunc('yearweek', s.published_at))
            ,p.id
            ,e.id
    """).df()
    DB.close()

    results = []
    for (emotion_id, publisher_id), group in df.groupby(['emotion_id', 'publisher_id']):
        model = linear_model.LinearRegression()
        x = group['date'].to_numpy().reshape(-1, 1)
        y = group['stories'].to_numpy()
        model.fit(x, y)
        per_year = model.coef_.item() * 60 * 60 * 24 * 365
        results.append({'emotion_id' : emotion_id, 'publisher_id':publisher_id, 'per_year' : per_year})
    results = pd.DataFrame(results)

    DB = connect()
    out = DB.query("""
        SELECT
            e.label as emotion
            --,p.tld
            ,avg(results.per_year) as avg_reg_coef
            ,b.ordinal
        FROM results
        JOIN emotions e
        ON e.id = results.emotion_id
        JOIN top.publishers p
        ON p.id = results.publisher_id
        JOIN publisher_bias pb
        ON pb.publisher_id = results.publisher_id
        JOIN bias_ratings b
        ON b.id = pb.bias_id
        GROUP BY
            e.label
            ,b.ordinal
    """).df()
    DB.close()
    pivot = out.pivot(index=['emotion'], columns=['ordinal'], values=['avg_reg_coef'])

    ax = sns.heatmap(pivot, cmap='RdBu_r')
    ticklabels = ['left', 'left-center', 'center', 'right-center', 'right']
    ax.set(title="slope of regression (stories/year) by bias and emotion"
           ,xticklabels=ticklabels
           ,xlabel="bias"
           ,ylabel="emotion")
    plt.tight_layout()
    plt.savefig(out_path / filename)
    print(f"saved: {filename}")

@click.command('plot:emotion-hist')
def emotion_hist():
    filename = "emotion_hist.png"

    DB = connect()
    DB.query("""describe story_emotions""")

    DB.query("""
        select
            e.label
            ,count(distinct s.id) as stories
            ,count(distinct s.publisher_id) as publishers
        from story_emotions se
        join emotions e
        on e.id = se.emotion_id
        join top.stories s
        on s.id = se.story_id
        group by
            e.label
    """).df().to_markdown(index=False)

    data = DB.sql("""
        SELECT
            b.ordinal
            ,count(1) as stories
        FROM stories s
        JOIN publisher_bias pb
        ON pb.publisher_id = s.publisher_id
        JOIN bias_ratings b
        ON b.id = pb.bias_id
        GROUP BY
            b.ordinal
    """).df()
    DB.close()

    ax = sns.barplot(x=data['ordinal'], y=data['stories'], color='tab:blue')
    ticklabels = ['left', 'left-center', 'center', 'right-center', 'right']
    ax.set(title="count of stories per bias rating", xlabel="bias rating", xticklabels=ticklabels)
    plt.tight_layout()
    plt.savefig(out_path / filename)
    print(f"saved: {filename}")
