import click
from data.main import connect
from links import to_matrix
import os
import seaborn as sns
import matplotlib.pyplot as plt
from pathlib import Path
import numpy as np
from sklearn.metrics import silhouette_score
import pandas as pd

out_dir = Path(os.getenv('DATA_MINING_DOC_DIR')) / 'figures'

@click.command('plot:link-elbow')
def elbow():
    from sklearn.cluster import KMeans

    filename = 'link_cluster_elbow.png'

    DB = connect()
    df = DB.query("""
        SELECT
            *
        FROM link_edges
    """).df()
    pivot = df.pivot(index='parent_id', columns='child_id', values='links').fillna(0)

    to_plot = []
    for k in range(2, 15):
        kmeans = KMeans(n_clusters=k, n_init="auto")
        kmeans.fit(pivot)
        label = kmeans.labels_
        coeff = silhouette_score(pivot, label, metric='euclidean')
        to_plot.append({'k': k, 'inertia' : kmeans.inertia_, 'coeff': coeff})
    to_plot = pd.DataFrame(to_plot)

    ax = sns.lineplot(x=to_plot.k, y=to_plot.inertia)
    ax.set(title="elbow criterion plot of clusters", xlabel="bin size (k)", ylabel="sum of squared distances between centroids/points")
    plt.savefig(out_dir / filename)
    plt.close()

    # randomly pick 8

@click.command('plot:link-pca-clusters')
@click.option('--source', type=click.Choice(['links', 'normalized', 'onehot']), default='links')
def link_pca_clusters(source):

    filename = f"link_pca_clusters_{source}.png"

    DB = connect()
    df = DB.query(f"""
        SELECT
            c.label as cluster
            ,p.tld
            --,b.label as bias
            ,pca.first
            ,pca.second
            ,s.cnt as stories
        FROM top.publisher_clusters_{source} c
        JOIN top.publishers p
        ON c.publisher_id = p.id
        JOIN 
        (
            select
                s.publisher_id
                ,count(1) as cnt
            FROM top.stories s
            GROUP BY
                s.publisher_id
        ) s
        ON s.publisher_id = p.id
        JOIN top.publisher_pca_{source} pca
        ON pca.publisher_id = p.id
    """).df()
    DB.close()

    ax = sns.scatterplot(x=df['first'], y=df['second'], hue=df['cluster'])
    ax.set(title=f"pca components vs. clusters ({source})", xlabel="first pca component", ylabel="second pca component")
    plt.savefig(out_dir / filename)

    # .df().groupby(['cluster', 'bias']).describe()




def test():
    data_dir = Path(os.getenv('DATA_MINING_DATA_DIR'))

    DB.query("""
        SELECT
            p.id as publisher_id
            ,p.name
            ,p.tld
            ,cast(b.bias_id as int) as bias_id
            ,count(1) as stories
        FROM publishers p
        JOIN stories s
        ON s.publisher_id = p.id
        JOIN publisher_clusters c
        ON c.publisher_id = p.id
        LEFT JOIN publisher_bias b
        ON b.publisher_id = p.id
        where bias_id is null
        group by
            p.id
            ,p.name
            ,p.tld
            ,b.bias_id
        ORDER BY count(1) desc
    """)

    # .df().to_csv(data_dir / 'cluster_publishers.csv', sep="|", index=False)
    DB.close()


@click.command('plot:link-confusion')
def link_confusion():
    from sklearn.model_selection import train_test_split
    from sklearn.neighbors import KNeighborsClassifier
    from sklearn.metrics import ConfusionMatrixDisplay

    filename = "link_confusion.png"

    DB = connect()
    bias = DB.query("""
        SELECT
            p.id as publisher_id
            ,b.ordinal
        FROM top.publishers p
        JOIN top.publisher_bias pb
        ON pb.publisher_id = p.id
        JOIN bias_ratings b
        ON b.id = pb.bias_id
    """).df()

    df = DB.query("""
        SELECT
            *
        FROM top.link_edges
        WHERE parent_id in (
            select
                publisher_id
            from bias
        )
        AND child_id in (
            select
                publisher_id
            from bias
        )
    """).df()
    pivot = df.pivot(index='parent_id', columns='child_id', values='links').fillna(0)

    x = pivot.values
    y = bias.sort_values('publisher_id').ordinal


    x_train, x_test = train_test_split(x)
    y_train, y_test = train_test_split(y)

    model = KNeighborsClassifier(n_neighbors=5)
    model.fit(x_train, y_train)
    y_pred = model.predict(x_test)


    fig, ax = plt.subplots(figsize=(10, 5))
    ConfusionMatrixDisplay.from_predictions(y_test, y_pred, ax=ax)
    ticklabels = ['left', 'left-center', 'center', 'right-center', 'right']
    ax.set(title="confusion matrix for kNN classifier on test data.", xticklabels=ticklabels, yticklabels=ticklabels)
    plt.savefig(out_dir / filename)
    plt.close()
    print(f"saved plot: {filename}")

@click.command('plot:link-classifier')
def link_confusion():
    from sklearn.model_selection import train_test_split
    from sklearn.neighbors import KNeighborsClassifier
    from sklearn.metrics import ConfusionMatrixDisplay

    filename = "link_confusion.png"

    DB = connect()
    bias = DB.query("""
        SELECT
            p.id as publisher_id
            ,b.ordinal
        FROM top.publishers p
        JOIN top.publisher_bias pb
        ON pb.publisher_id = p.id
        JOIN bias_ratings b
        ON b.id = pb.bias_id
    """).df()

    df = DB.query("""
        SELECT
            *
        FROM top.link_edges
        WHERE parent_id in (
            select
                publisher_id
            from bias
        )
        AND child_id in (
            select
                publisher_id
            from bias
        )
    """).df()
    pivot = df.pivot(index='parent_id', columns='child_id', values='links').fillna(0)

    x = pivot.values
    y = bias.sort_values('publisher_id').ordinal

    data = DB.query(f"""
        SELECT
            p.id as publisher_id
            ,pca.first
            ,pca.second
        FROM top.publisher_pca_onehot pca
        JOIN top.publishers p
        ON pca.publisher_id = p.id
    """).df()



    model = KNeighborsClassifier(n_neighbors=5)
    model.fit(x, y)
    y_pred = model.predict(x)

    plot = bias.sort_values('publisher_id')
    plot['pred'] = y_pred
    data = pd.merge(plot, data)


    fig, ax = plt.subplots(figsize=(10, 5))
    ConfusionMatrixDisplay.from_predictions(data['ordinal'], data['pred'], ax=ax)
    ticklabels = ['left', 'left-center', 'center', 'right-center', 'right']
    ax.set(title="confusion matrix for link matrix kNN classifier", xticklabels=ticklabels, yticklabels=ticklabels)
    plt.savefig(out_dir / filename)
    plt.close()
    print(f"saved plot: {filename}")

    ax = sns.scatterplot(x=data['first'], y=data['second'], hue=data['pred'])
    plt.savefig(out_dir / filename)
    plt.close()
    print(f"saved plot: {filename}")
