import click
from data.main import connect
import os
from pathlib import Path
import seaborn as sns
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd

out_path = Path(os.getenv('DATA_MINING_DOC_DIR')) / 'figures'
data_path = Path(os.getenv('DATA_MINING_DATA_DIR'))

@click.command('plot:sentence-pca')
def sentence_pca():
    filename = "embedding_sentence_pca.png"
    DB = connect()

    data = DB.query("""
        SELECT
            pca.first
            ,pca.second
            ,b.bias as label
        FROM top.story_embeddings_pca pca
        JOIN top.stories s
        ON s.id = pca.story_id
        JOIN top.publisher_bias pb
        ON pb.publisher_id = s.publisher_id
        JOIN bias_ratings b
        ON b.id = pb.bias_id
    """).df()
    DB.close()

    ax = sns.scatterplot(x=data['first'], y=data['second'], hue=data['label'])
    ax.set(title="pca components vs. bias label", xlabel="first component", ylabel="second component")
    plt.savefig(out_path / filename)

@click.command('plot:avg-sentence-pca')
def avg_sentence_pca():
    filename = "avg_embedding_sentence_pca.png"
    DB = connect()

    data = DB.query("""
        SELECT
            pca.first
            ,pca.second
            ,p.tld
            ,b.bias as label
        FROM top.publisher_embeddings_pca pca
        JOIN top.publishers p
        ON p.id = pca.publisher_id
        JOIN top.publisher_bias pb
        ON pb.publisher_id = p.id
        JOIN bias_ratings b
        ON b.id = pb.bias_id
    """).df()
    DB.close()

    ax = sns.scatterplot(x=data['first'], y=data['second'], hue=data['label'])
    ax.set(title="avg. publisher embedding pca components vs. bias label", xlabel="first component", ylabel="second component")
    plt.savefig(out_path / filename)

@click.command('plot:sentence-confusion')
def sentence_confusion():
    from sklearn.model_selection import train_test_split
    from sklearn.neighbors import KNeighborsClassifier
    from sklearn.metrics import ConfusionMatrixDisplay

    filename = "sentence_confusion.png"

    embeddings = np.load(data_path / 'embeddings.npy')
    embedding_ids = np.load(data_path / 'embedding_ids.npy')
    ids = pd.DataFrame(embedding_ids, columns=['story_id']).reset_index()

    DB = connect()
    data = DB.query("""
        SELECT
            ids.index
            ,s.id
            ,b.ordinal
        FROM ids
        JOIN top.stories s
        ON ids.story_id = s.id
        JOIN top.publisher_bias pb
        ON pb.publisher_id = s.publisher_id
        JOIN bias_ratings b
        ON b.id = pb.bias_id
    """).df()
    pub = DB.query("""
        SELECT
            *
        FROM top.publishers
    """).df()
    DB.close()

    train, test = train_test_split(data)
    train_x, train_y = embeddings[train['index']], train['ordinal']
    test_x, test_y = embeddings[test['index']], test['ordinal']

    model = KNeighborsClassifier(n_neighbors=5)
    model.fit(train_x, train_y)
    pred = model.predict(test_x)


    fig, ax = plt.subplots(figsize=(10, 5))
    ConfusionMatrixDisplay.from_predictions(test_y, pred, ax=ax)
    ticklabels = ['left', 'left-center', 'center', 'right-center', 'right']
    ax.set(title="confusion matrix for kNN classifier on test data.", xticklabels=ticklabels, yticklabels=ticklabels)
    plt.savefig(out_path / filename)
    plt.close()

    print(f"saved plot: {filename}")
