import click
from data.main import connect
import os
from pathlib import Path
import seaborn as sns
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd

out_path = Path(os.getenv('DATA_MINING_DOC_DIR')) / 'figures'

@click.command('plot:sentiment-over-time')
def over_time():
    filename = "sentiment_over_time.png"

    DB = connect()
    data = DB.sql("""
        SELECT
            avg(sent.class_id) as sentiment
            ,s.published_at as date
        FROM top.story_sentiments sent
        JOIN top.stories s
        ON s.id = sent.story_id
        GROUP BY
            s.published_at
    """).df()
    DB.close()

    ax = sns.scatterplot(x=data['date'], y=data['sentiment'])
    ax.set(title="sentiment vs. time")
    plt.tight_layout()
    plt.savefig(out_path / filename)
    print(f"saved: {filename}")

@click.command('plot:bias-vs-sentiment-over-time')
def bias_over_time():
    filename = "bias_vs_sentiment_over_time.png"

    DB = connect()
    data = DB.sql("""
        SELECT
            avg(sent.class_id) as sentiment
            ,date_trunc('yearweek', s.published_at) as date
            --,b.ordinal as ordinal
            ,b.bias
        FROM top.story_sentiments sent
        JOIN top.stories s
        ON s.id = sent.story_id
        JOIN publisher_bias pb
        ON pb.publisher_id = s.publisher_id
        JOIN bias_ratings b
        ON b.id = pb.bias_id
        GROUP BY
            date_trunc('yearweek', s.published_at)
            ,b.bias
    """).df()
    DB.close()

    order = ['left', 'left-center', 'center', 'right-center', 'right']
    ax = sns.relplot(data, x='date', y='sentiment', col='bias', col_order=order)
    plt.tight_layout()
    plt.savefig(out_path / filename)
    plt.close()
    print(f"saved: {filename}")

@click.command('plot:sentiment-recent-winner')
def bias_vs_recent_winner():
    filename = "bias_vs_recent_winner.png"

    DB = connect()
    data = DB.sql("""
        SELECT
            e.days_away as days_away
            ,b.ordinal
            ,avg(sent.class_id) as sentiment
            ,count(1) as stories
        FROM top.stories s
        JOIN top.story_sentiments sent
        ON s.id = sent.story_id
        JOIN election_distance e
        ON e.publish_date = s.published_at
        JOIN publisher_bias pb
        ON pb.publisher_id = s.publisher_id
        JOIN bias_ratings b
        ON b.id = pb.bias_id
        GROUP BY
            e.days_away
            ,b.ordinal
    """).df()
    DB.close()
    data

    ax = sns.scatterplot(x=data['days_away'], y=data['sentiment'], hue=data['ordinal'])
    ax.set(title="sentiment vs. days to nearest election", xlabel="days to nearest election", ylabel="average title seniment")
    plt.tight_layout()
    plt.savefig(out_path / filename)
    plt.close()

    print(f"saved: {filename}")

@click.command('plot:sentiment-hist')
def sentiment_hist():
    filename = "sentiment_hist.png"

    DB = connect()

    DB.query("""
        select
            sent.label
            ,count(distinct s.id) as stories
            ,count(distinct s.publisher_id) as publishers
        from top.story_sentiments sent
        join top.stories s
        on s.id = sent.story_id
        group by
            sent.label
    """).df().to_markdown(index=False)

    data = DB.sql("""
        SELECT
            b.ordinal
            ,count(1) as stories
        FROM stories s
        JOIN publisher_bias pb
        ON pb.publisher_id = s.publisher_id
        JOIN bias_ratings b
        ON b.id = pb.bias_id
        GROUP BY
            b.ordinal
    """).df()
    DB.close()

    ax = sns.barplot(x=data['ordinal'], y=data['stories'], color='tab:blue')
    ticklabels = ['left', 'left-center', 'center', 'right-center', 'right']
    ax.set(title="count of stories per bias rating", xlabel="bias rating", xticklabels=ticklabels)
    plt.tight_layout()
    plt.savefig(out_path / filename)
    print(f"saved: {filename}")
